#!/usr/bin/env node
'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _projectRunner = require('../projectRunner');

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _npid = require('npid');

var _npid2 = _interopRequireDefault(_npid);

var _minimist = require('minimist');

var _minimist2 = _interopRequireDefault(_minimist);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _readline = require('readline');

var _readline2 = _interopRequireDefault(_readline);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-console */

// import "source-map-support/register"
console.info('=== Fil ===');
console.info('Running using node ' + process.version);

var argv = (0, _minimist2.default)(process.argv, { boolean: ['dynamic', 'force', 'nocache'] });

var projectRootFile = require(_path2.default.join(process.cwd(), 'index.js'));
// noinspection JSUnresolvedVariable
var projectRunner = new _projectRunner.ProjectRunner({
  listenToChanges: argv.dynamic,
  project: projectRootFile,
  useCache: !argv.nocache
});

var pidFolder = _path2.default.join(process.cwd(), projectRootFile.cachePath);

_fsExtra2.default.ensureDirSync(pidFolder); // eslint-disable-line no-sync

var pid = _npid2.default.create(_path2.default.join(pidFolder, 'running.pid'), argv.force);

if (process.platform === 'win32') {
  var rl = _readline2.default.createInterface({
    input: process.stdin,
    output: process.stdout
  });

  rl.on('SIGINT', function () {
    process.emit('SIGINT');
  });
}

var cleanup = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            return _context.abrupt('return', projectRunner.persistCache().catch(function (e) {
              console.error(e);
              process.exitCode = 1;
            }).then(function () {
              pid.remove();
            }));

          case 1:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function cleanup() {
    return _ref.apply(this, arguments);
  };
}();

var sigintInProgress = false;
process.on('SIGINT', function () {
  if (sigintInProgress) {
    console.log('Dude, shutdown is already in progress... Get a faster machine or calm down.');
    return;
  }
  sigintInProgress = true;
  console.log('Preparing to shutdown...');
  cleanup().then(function () {
    // If we are in dynamic serving SIGINT is the correct way to exit.
    process.exit(argv.dynamic ? 0 : 4); // eslint-disable-line no-process-exit
  }).catch(function () {
    process.exit(3); // eslint-disable-line no-process-exit
  });
});

projectRunner.init().then(function () {
  if (argv.dynamic) {
    return projectRunner.generateDynamic().then(function () {
      return new _promise2.default(function () {
        // prevent resolving forever.
      });
    });
  }
  return projectRunner.generateStatic();
}).catch(function (e) {
  console.error(e);
  process.exitCode = 2;
}).then(cleanup).then(function () {
  if (process.exitCode !== 0) {
    process.exit(process.exitCode); // eslint-disable-line no-process-exit
  }
});