'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Project = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _cache = require('./utils/cache');

var _contentManager = require('./contentManager');

var _routeManager = require('./routeManager');

var _Rx = require('rxjs/Rx');

var _Rx2 = _interopRequireDefault(_Rx);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _misc = require('./utils/misc');

var _v = require('uuid/v1');

var _v2 = _interopRequireDefault(_v);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('fil:project');

var Project = exports.Project = function () {
  function Project(_ref) {
    var listenToChanges = _ref.listenToChanges,
        project = _ref.project,
        useCache = _ref.useCache;
    (0, _classCallCheck3.default)(this, Project);

    var contentTypes = (0, _keys2.default)(project.routeHandlers).map(function (key) {
      var handler = project.routeHandlers[key];
      var content = Project.routeHandler2RegularContent({ routeHandler: handler, type: key });
      return { content: content, key: key };
    }).reduce(function (acc, _ref2) {
      var content = _ref2.content,
          key = _ref2.key;

      if (acc[key]) {
        throw new Error('Duplicate key ' + key);
      }
      return (0, _assign2.default)((0, _defineProperty3.default)({}, key, content), acc);
    }, project.contentTypes || {});

    this._project = (0, _assign2.default)({}, project, { contentTypes: contentTypes });

    this._listenToChanges = listenToChanges;
    this._useCache = useCache;
    this._routeManager = new _routeManager.RouteManager({ project: this });
    this._contentManager = new _contentManager.ContentManager({ checkForChanges: listenToChanges, project: this });

    if (this._useCache) {
      this.cachePath = this._project.cachePath;
    } else {
      this.cachePath = _path2.default.join(_os2.default.tmpdir(), (0, _v2.default)());
    }
  }

  /* init & dispose logic */


  (0, _createClass3.default)(Project, [{
    key: 'initCache',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        var cachePath, _ref4, _ref5, currentHash, cachedHash;

        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (this._useCache) {
                  _context.next = 3;
                  break;
                }

                debug('Cache will not be used');
                return _context.abrupt('return');

              case 3:
                debug('Checking if project is changed while fil is not running');
                cachePath = this._project.cachePath;
                _context.next = 7;
                return _promise2.default.all([this._project.contentVersion(), (0, _cache.readHash)({ cachePath: cachePath }).catch(function () {
                  return null;
                })]);

              case 7:
                _ref4 = _context.sent;
                _ref5 = (0, _slicedToArray3.default)(_ref4, 2);
                currentHash = _ref5[0];
                cachedHash = _ref5[1];

                debug('Current: ' + currentHash + ' Cached: ' + cachedHash);

                if (!(currentHash !== cachedHash)) {
                  _context.next = 18;
                  break;
                }

                debug('Cache will be ignored.');
                _context.next = 16;
                return (0, _cache.clearCache)({ cachePath: cachePath });

              case 16:
                debug('Deleted up obsolete cache data... (' + cachePath + ')');
                return _context.abrupt('return');

              case 18:

                debug('Loading cache...');
                _context.next = 21;
                return this._contentManager.loadCache();

              case 21:
                debug('Cache ready!');

              case 22:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function initCache() {
        return _ref3.apply(this, arguments);
      }

      return initCache;
    }()

    /* Route related stuff */

  }, {
    key: 'handledUrls',
    value: function () {
      var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                return _context2.abrupt('return', this._routeManager.handledUrls());

              case 1:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function handledUrls() {
        return _ref6.apply(this, arguments);
      }

      return handledUrls;
    }()
  }, {
    key: 'handle',
    value: function () {
      var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(_ref8) {
        var url = _ref8.url;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                return _context3.abrupt('return', this._routeManager.handle({ url: url }));

              case 1:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function handle(_x) {
        return _ref7.apply(this, arguments);
      }

      return handle;
    }()
  }, {
    key: 'handleAll',
    value: function () {
      var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(_ref10) {
        var urlProcessFn = _ref10.urlProcessFn;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                return _context4.abrupt('return', this._routeManager.handleAll({ urlProcessFn: urlProcessFn }));

              case 1:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function handleAll(_x2) {
        return _ref9.apply(this, arguments);
      }

      return handleAll;
    }()

    /* Content related stuff */

  }, {
    key: 'contentTypes',
    value: function contentTypes() {
      // Returns an array of registered types
      return this._contentManager.contentTypes();
    }
  }, {
    key: 'metaOf',
    value: function () {
      var _ref11 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(_ref12) {
        var id = _ref12.id,
            type = _ref12.type;
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                return _context5.abrupt('return', this._contentManager.metaOf({ id: id, type: type }));

              case 1:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function metaOf(_x3) {
        return _ref11.apply(this, arguments);
      }

      return metaOf;
    }()
  }, {
    key: 'valueOf',
    value: function () {
      var _ref13 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(_ref14) {
        var _data = _ref14._data,
            id = _ref14.id,
            type = _ref14.type;
        return _regenerator2.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                return _context6.abrupt('return', this._contentManager.valueOf({ _data: _data, id: id, type: type }));

              case 1:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function valueOf(_x4) {
        return _ref13.apply(this, arguments);
      }

      return valueOf;
    }()

    /* Cache persistence */

  }, {
    key: 'persistCache',
    value: function () {
      var _ref15 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7() {
        var cachePath, hash;
        return _regenerator2.default.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                if (this._useCache) {
                  _context7.next = 2;
                  break;
                }

                return _context7.abrupt('return');

              case 2:
                debug('Persisting cache');

                cachePath = this._project.cachePath;
                _context7.next = 6;
                return this._project.contentVersion();

              case 6:
                hash = _context7.sent;

                debug('Content hash: ' + hash);

                return _context7.abrupt('return', _promise2.default.all([(0, _cache.writeHash)({ cachePath: cachePath, hash: hash }), this._contentManager.persistCache()]));

              case 9:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function persistCache() {
        return _ref15.apply(this, arguments);
      }

      return persistCache;
    }()

    /* The ultimate change detector */

  }, {
    key: 'watcher$',
    value: function watcher$() {
      if (!this._listenToChanges) {
        return _Rx2.default.Observable.empty();
      }
      return _Rx2.default.Observable.merge(this._contentManager.watcher$(), (0, _misc.toObservable)({ fn: this._project.watcher }));
    }

    /* Path resolving */

  }, {
    key: 'outPath',
    value: function outPath() {
      return this._project.outPath;
    }
  }]);
  return Project;
}();

Project.routeHandler2RegularContent = function (_ref16) {
  var type = _ref16.type,
      routeHandler = _ref16.routeHandler;

  var handleFn = routeHandler.handle;
  var handlesFn = routeHandler.handles;
  var handleWatcherFn = routeHandler.handleWatcher;
  var handlesWatcherFn = routeHandler.handlesWatcher;

  return {
    children: function () {
      var _ref17 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(_ref18) {
        var project = _ref18.project;
        var urls;
        return _regenerator2.default.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                _context8.next = 2;
                return handlesFn({ project: project });

              case 2:
                urls = _context8.sent;
                return _context8.abrupt('return', urls.map(function (url) {
                  var returnValue = { type: type };
                  if (typeof url === 'string') {
                    returnValue.id = url;
                  } else {
                    returnValue.id = url.url;
                    returnValue._data = url.data; // eslint-disable-line no-underscore-dangle
                  }
                  return returnValue;
                }));

              case 4:
              case 'end':
                return _context8.stop();
            }
          }
        }, _callee8, undefined);
      }));

      return function children(_x5) {
        return _ref17.apply(this, arguments);
      };
    }(),
    childrenWatcher: handlesWatcherFn,
    content: function content(_ref19) {
      var _data = _ref19._data,
          id = _ref19.id,
          project = _ref19.project;
      return handleFn({ data: _data, project: project, url: id });
    },
    contentWatcher: handleWatcherFn ? function (_ref20) {
      var id = _ref20.id,
          notifyFn = _ref20.notifyFn;
      return handleWatcherFn({ notifyFn: notifyFn, url: id });
    } : null
  };
};