'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ReactiveCache = undefined;

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _cache = require('./utils/cache');

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debugCacheMiss = (0, _debug2.default)('fil:cache:miss');
var debugChangeDetected = (0, _debug2.default)('fil:cache:change');
var debugWatchForChanges = (0, _debug2.default)('fil:cache:watchForChanges');

var ReactiveCache = exports.ReactiveCache = function () {
  function ReactiveCache() {
    (0, _classCallCheck3.default)(this, ReactiveCache);

    this._cache = {};
  }

  (0, _createClass3.default)(ReactiveCache, [{
    key: 'calculate',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(_ref2) {
        var id = _ref2.id,
            valueFn = _ref2.valueFn,
            watchFn = _ref2.watchFn,
            notifyFn = _ref2.notifyFn;
        var value;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!this._cache[id]) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt('return', this._cache[id].value);

              case 2:

                debugCacheMiss(id);
                _context.next = 5;
                return valueFn();

              case 5:
                value = _context.sent;

                this._setAndWatchCache({ id: id, notifyFn: notifyFn, value: value, watchFn: watchFn });
                return _context.abrupt('return', value);

              case 8:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function calculate(_x) {
        return _ref.apply(this, arguments);
      }

      return calculate;
    }()
  }, {
    key: 'remove',
    value: function remove(_ref3) {
      var id = _ref3.id;

      var cachedItem = this._cache[id];
      if (!cachedItem) {
        return null;
      }

      var unsubscribeFn = cachedItem.unsubscribeFn,
          value = cachedItem.value;

      unsubscribeFn();
      delete this._cache[id];
      return value;
    }
  }, {
    key: 'persistCache',
    value: function () {
      var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(_ref5) {
        var _this = this;

        var cachePath = _ref5.cachePath,
            key = _ref5.key;
        var values;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                values = (0, _keys2.default)(this._cache).map(function (id) {
                  var value = _this._cache[id].value;
                  return { id: id, value: value };
                });
                return _context2.abrupt('return', (0, _cache.writeObject)({
                  cachePath: cachePath,
                  key: key,
                  object: values
                }));

              case 2:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function persistCache(_x2) {
        return _ref4.apply(this, arguments);
      }

      return persistCache;
    }()
  }, {
    key: 'loadCache',
    value: function () {
      var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(_ref7) {
        var _this2 = this;

        var cachePath = _ref7.cachePath,
            key = _ref7.key,
            functionLoaderFn = _ref7.functionLoaderFn;
        var values;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return (0, _cache.readObject)({
                  cachePath: cachePath,
                  key: key
                });

              case 2:
                values = _context3.sent;

                if (!(values === null)) {
                  _context3.next = 5;
                  break;
                }

                return _context3.abrupt('return');

              case 5:

                values.forEach(function (_ref8) {
                  var id = _ref8.id,
                      value = _ref8.value;

                  var _functionLoaderFn = functionLoaderFn({ id: id, value: value }),
                      watchFn = _functionLoaderFn.watchFn,
                      notifyFn = _functionLoaderFn.notifyFn;

                  _this2._setAndWatchCache({ id: id, notifyFn: notifyFn, value: value, watchFn: watchFn });
                });

              case 6:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function loadCache(_x3) {
        return _ref6.apply(this, arguments);
      }

      return loadCache;
    }()
  }, {
    key: '_setAndWatchCache',
    value: function _setAndWatchCache(_ref9) {
      var id = _ref9.id,
          value = _ref9.value,
          watchFn = _ref9.watchFn,
          notifyFn = _ref9.notifyFn;

      debugWatchForChanges(id);
      var externalNotifyFn = this._onValueChanged.bind(this, { id: id });
      var retVal = watchFn({ notifyFn: externalNotifyFn });
      var unsubscribeFn = retVal && retVal.unsubscribe ? retVal.unsubscribe : function () {
        return {};
      };

      this._cache[id] = {
        notifyFn: notifyFn,
        unsubscribeFn: unsubscribeFn,
        value: value
      };
    }
  }, {
    key: '_onValueChanged',
    value: function _onValueChanged(_ref10) {
      var id = _ref10.id;

      var cachedItem = this._cache[id];
      if (!cachedItem) {
        return;
      }

      debugChangeDetected(id);
      var unsubscribeFn = cachedItem.unsubscribeFn,
          notifyFn = cachedItem.notifyFn,
          value = cachedItem.value;

      delete this._cache[id];
      unsubscribeFn();
      notifyFn({ oldValue: value });
    }
  }]);
  return ReactiveCache;
}();