'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.writeObject = exports.readObject = exports.clearCache = exports.writeHash = exports.readHash = undefined;

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _misc = require('./misc');

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _v = require('uuid/v1');

var _v2 = _interopRequireDefault(_v);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var serializedValuePrefix = '@@SERIALIZED@@';
var hashFileName = 'hash.txt';

var readHash = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(_ref2) {
    var cachePath = _ref2.cachePath;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            return _context.abrupt('return', _fsExtra2.default.readFile(_path2.default.join(cachePath, hashFileName), 'utf-8'));

          case 1:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function readHash(_x) {
    return _ref.apply(this, arguments);
  };
}();
var writeHash = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(_ref4) {
    var cachePath = _ref4.cachePath,
        hash = _ref4.hash;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            return _context2.abrupt('return', _fsExtra2.default.outputFile(_path2.default.join(cachePath, hashFileName), hash));

          case 1:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  }));

  return function writeHash(_x2) {
    return _ref3.apply(this, arguments);
  };
}();
var clearCache = function () {
  var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(_ref6) {
    var cachePath = _ref6.cachePath;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            return _context3.abrupt('return', _fsExtra2.default.emptyDir(cachePath));

          case 1:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  }));

  return function clearCache(_x3) {
    return _ref5.apply(this, arguments);
  };
}();

var pathForCacheItem = function pathForCacheItem(_ref7) {
  var uuid = _ref7.uuid;
  return _path2.default.join(uuid[0], uuid[1], uuid[2], uuid[3], uuid[4], uuid);
};

var writeObject = function () {
  var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(_ref9) {
    var key = _ref9.key,
        object = _ref9.object,
        cachePath = _ref9.cachePath;
    var serializedFieldsInfo, toDisk, json, finalJson, fullPath;
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            if (object) {
              _context4.next = 2;
              break;
            }

            return _context4.abrupt('return');

          case 2:

            // uuid, type, value
            serializedFieldsInfo = [];

            // name: relative file path in cache; buffer: actual buffer to be written

            toDisk = [];
            json = (0, _misc.deepMap)(object, function (value) {
              if (value instanceof Buffer) {
                var uuid = (0, _v2.default)();
                var filePath = pathForCacheItem({ uuid: uuid });
                toDisk.push({
                  buffer: value,
                  name: filePath
                });
                serializedFieldsInfo.push({
                  type: 'buffer',
                  uuid: uuid,
                  value: filePath
                });
                return '' + serializedValuePrefix + uuid;
              }

              if (value instanceof Date) {
                var _uuid = (0, _v2.default)();
                serializedFieldsInfo.push({
                  type: 'date',
                  uuid: _uuid,
                  value: value.toJSON()
                });
                return '' + serializedValuePrefix + _uuid;
              }

              return value;
            });
            _context4.next = 7;
            return _promise2.default.all(toDisk.map(function (_ref10) {
              var name = _ref10.name,
                  buffer = _ref10.buffer;

              var fullPath = _path2.default.join(cachePath, name);
              return _fsExtra2.default.outputFile(fullPath, buffer);
            }));

          case 7:
            finalJson = {
              json: json,
              serializedFieldsInfo: serializedFieldsInfo
            };
            fullPath = _path2.default.join(cachePath, key + '.json');
            _context4.next = 11;
            return _fsExtra2.default.outputJson(fullPath, finalJson);

          case 11:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  }));

  return function writeObject(_x4) {
    return _ref8.apply(this, arguments);
  };
}();

var readObject = function () {
  var _ref11 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(_ref12) {
    var key = _ref12.key,
        cachePath = _ref12.cachePath;
    var filePath, fileContents, serializedFieldsInfo, cacheObject;
    return _regenerator2.default.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            filePath = _path2.default.join(cachePath, key + '.json');
            fileContents = null;
            _context6.prev = 2;
            _context6.next = 5;
            return _fsExtra2.default.readFile(filePath, 'utf8');

          case 5:
            fileContents = _context6.sent;

            fileContents = JSON.parse(fileContents);
            _context6.next = 12;
            break;

          case 9:
            _context6.prev = 9;
            _context6.t0 = _context6['catch'](2);
            return _context6.abrupt('return', null);

          case 12:
            _context6.next = 14;
            return _promise2.default.all(fileContents.serializedFieldsInfo.map(function () {
              var _ref13 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(_ref14) {
                var uuid = _ref14.uuid,
                    value = _ref14.value,
                    type = _ref14.type;
                var fullPath, fcontents;
                return _regenerator2.default.wrap(function _callee5$(_context5) {
                  while (1) {
                    switch (_context5.prev = _context5.next) {
                      case 0:
                        if (!(type === 'buffer')) {
                          _context5.next = 6;
                          break;
                        }

                        fullPath = _path2.default.join(cachePath, value);
                        _context5.next = 4;
                        return _fsExtra2.default.readFile(fullPath);

                      case 4:
                        fcontents = _context5.sent;
                        return _context5.abrupt('return', { type: type, uuid: uuid, value: fcontents });

                      case 6:
                        if (!(type === 'date')) {
                          _context5.next = 8;
                          break;
                        }

                        return _context5.abrupt('return', { type: type, uuid: uuid, value: new Date(value) });

                      case 8:
                        throw new Error('Unknown type ' + type + ' on cache uuid ' + uuid);

                      case 9:
                      case 'end':
                        return _context5.stop();
                    }
                  }
                }, _callee5, undefined);
              }));

              return function (_x6) {
                return _ref13.apply(this, arguments);
              };
            }()));

          case 14:
            _context6.t1 = function (acc, sfi) {
              return (0, _assign2.default)({}, acc, (0, _defineProperty3.default)({}, sfi.uuid, {
                type: sfi.type,
                value: sfi.value
              }));
            };

            _context6.t2 = {};
            serializedFieldsInfo = _context6.sent.reduce(_context6.t1, _context6.t2);
            cacheObject = (0, _misc.deepMap)(fileContents.json, function (value) {
              if (!(typeof value === 'string' && value.startsWith(serializedValuePrefix))) {
                return value;
              }
              var uuid = value.substr(serializedValuePrefix.length);
              return serializedFieldsInfo[uuid].value;
            });
            return _context6.abrupt('return', cacheObject);

          case 19:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, undefined, [[2, 9]]);
  }));

  return function readObject(_x5) {
    return _ref11.apply(this, arguments);
  };
}();

exports.readHash = readHash;
exports.writeHash = writeHash;
exports.clearCache = clearCache;
exports.readObject = readObject;
exports.writeObject = writeObject;